#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_fuzzyfinder
----------------------------------

Tests for `fuzzyfinder` module.
"""
import pytest
from fuzzyfinder import fuzzyfinder

@pytest.fixture
def collection():
    return ['migrations.py',
            'django_migrations.py',
            'django_admin_log.py',
            'api_user.doc',
            'user_group.doc',
            'users.txt',
            'accounts.txt',
            '123.py',
            'test123test.py'
            ]

@pytest.fixture
def cased_collection():
    return ['MIGRATIONS.py',
            'django_MiGRations.py',
            'django_admin_log.py',
            'migrations.doc',
            'user_group.doc',
            'users.txt',
            ]

@pytest.fixture
def dict_collection():
    return [
            {'name': 'migrations.py'},
            {'name': 'django_migrations.py'},
            {'name': 'django_admin_log.py'},
            {'name': 'api_user.doc'},
            {'name': 'user_group.doc'},
            {'name': 'users.txt'},
            {'name': 'accounts.txt'},
            ]

def test_substring_match(collection):
    text = 'txt'
    results = fuzzyfinder(text, collection)
    expected = ['users.txt', 'accounts.txt']
    assert list(results) == expected

def test_case_insensitive_substring_match(cased_collection):
    text = 'miGr'
    results = fuzzyfinder(text, cased_collection)
    expected = ['MIGRATIONS.py', 'migrations.doc', 'django_MiGRations.py']
    assert list(results) == expected

def test_substring_match_with_dot(collection):
    text = '.txt'
    results = fuzzyfinder(text, collection)
    expected = ['users.txt', 'accounts.txt']
    assert list(results) == expected

def test_fuzzy_match(collection):
    text = 'djmi'
    results = fuzzyfinder(text, collection)
    expected = ['django_migrations.py', 'django_admin_log.py']
    assert list(results) == expected

def test_fuzzy_match_ranking(collection):
    text = 'mi'
    results = fuzzyfinder(text, collection)
    expected = ['migrations.py', 'django_migrations.py', 'django_admin_log.py']
    assert list(results) == expected

def test_fuzzy_match_greedy(collection):
    text = 'user'
    results = fuzzyfinder(text, collection)
    expected = ['user_group.doc', 'users.txt', 'api_user.doc']
    assert list(results) == expected

def test_fuzzy_integer_input(collection):
    text = 123
    results = fuzzyfinder(text, collection)
    expected = ['123.py', 'test123test.py']

def test_accessor(dict_collection):
    text = 'user'
    results = fuzzyfinder(text, dict_collection, lambda x: x['name'])
    expected = [{'name': 'user_group.doc'}, {'name': 'users.txt'}, {'name': 'api_user.doc'}]
    assert list(results) == expected



# -*- coding: utf-8 -*-

__author__ = 'Amjith Ramanujam'
__email__ = 'amjith.r@gmail.com'
__version__ = '2.0.0'

__all__ = []

def export(defn):
    """Decorator to explicitly mark functions that are exposed in a lib."""
    globals()[defn.__name__] = defn
    __all__.append(defn.__name__)
    return defn

from . import main



# -*- coding: utf-8 -*-
import re
from . import export

@export
def fuzzyfinder(input, collection, accessor=lambda x: x):
    """
    Args:
        input (str): A partial string which is typically entered by a user.
        collection (iterable): A collection of strings which will be filtered
                               based on the `input`.

    Returns:
        suggestions (generator): A generator object that produces a list of
            suggestions narrowed down from `collection` using the `input`.
    """
    suggestions = []
    input = str(input) if not isinstance(input, str) else input
    pat = '.*?'.join(map(re.escape, input))
    regex = re.compile(pat, re.IGNORECASE)
    for item in collection:
        r = regex.search(accessor(item))
        if r:
            suggestions.append((len(r.group()), r.start(), accessor(item), item))

    return (z[-1] for z in sorted(suggestions))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
import ast
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('fuzzyfinder/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
        ]

test_requirements = [
    'pytest',
]

setup(
    name='fuzzyfinder',
    version=version,
    description="Fuzzy Finder implemented in Python.",
    long_description=readme + '\n\n' + history,
    author="Amjith Ramanujam",
    author_email='amjith.r@gmail.com',
    url='https://github.com/amjith/fuzzyfinder',
    packages=[
        'fuzzyfinder',
    ],
    package_dir={'fuzzyfinder':
                 'fuzzyfinder'},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='fuzzyfinder',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ]
)
